CLI_CONFIG = {
    "output": {
        "source": "rend",
        "default": None,
    },
    "ref": {"positional": True},
}

CONFIG = {
    "ref": {
        "type": str,
        "help": "The sub on the hub to parse",
        "default": None,
    },
}

DYNE = {"doc": ["doc"]}
